@@ -7,7 +7,8 @@
 @created_time: 2022/9/5 11:33
 """
 
-from django.core.paginator import Paginator
+from django.core.paginator import Paginator, EmptyPage
+from django.db import transaction
 from django.db.models import F
 from django.db.models import Q
 
@@ -16,6 +17,7 @@ from xj_user.services.user_service import UserService
 from ..models import UserToGroup, RoleUserGroup, UserToRole
 from ..utils.custom_tool import format_params_handle, parse_json, format_list_handle
 from ..utils.j_recur import JRecur
+from ..utils.join_list import JoinList
 
 
 # 用户组CURD服务
@@ -100,42 +102,95 @@ class UserGroupService(object):
         group_set = RoleUserGroup.objects.filter(**params)
         count = group_set.count()
         group_list = group_set.values()
-        finish_set = list(Paginator(group_list, size).page(page))
-        return {"page": int(page), "size": int(size), "count": int(count), "list": finish_set}, None
+        paginator = Paginator(group_list, size)
+        try:
+            finish_set = paginator.page(page)
+        except EmptyPage:
+            finish_set = paginator.page(paginator.num_pages)
+
+        return {"page": int(page), "size": int(size), "count": int(count), "list": list(finish_set.object_list)}, None
 
     @staticmethod
-    def user_group_users(group_id, page, size, is_subtree=False):
+    def get_user_ids_by_group(group_id):
         """
-        用户分组下用户列表
-        :param group_id: 分组id
-        :param page: 页数
-        :param size: 条数
-        :return:
+        根据用户组ID,获取该分组下面的所有用户ID列表
+        :param group_id: 分组ID
+        :return: data, err
         """
-        if is_subtree:
-            group_id_tree, err = UserGroupService.get_user_group_tree(group_id)
+        user_obj = UserToGroup.objects.filter(user_group_id=group_id)
+        user_list = []
+        if user_obj:
+            user_list = user_obj.values("user_id")
+            user_list = [i['user_id'] for i in user_list]
+        return user_list, None
+
+    @staticmethod
+    def in_group_users(params=None, need_child=False):
+        """
+        获取用户分组下用户列表
+        :param params: 查询参数
+        :param need_child:  是否查询子节点
+        :return: data, err
+        """
+        if params is None:
+            params = {}
+        # 分页参数
+        page = params.pop("page", 1)
+        size = params.pop("size", 10)
+
+        # 用户条件反查, 得到用user_id列表.
+        user_search_params = format_params_handle(
+            param_dict=params,
+            filter_filed_list=["phone", "user_name", "full_name", "email"],
+            is_remove_empty=True
+        )
+        if user_search_params:
+            user_serv, err = UserService.user_list(params=user_search_params, need_Pagination=False)
+            params["user_id_list"] = [i["user_id"] for i in user_serv if i]
+
+        # 是否需要子节点搜索
+        user_group_id = params.get("user_group_id") or params.get("group_id")
+        if need_child and user_group_id:
+            group_id_tree, err = UserGroupService.get_user_group_tree(user_group_id)
             if err:
                 return None, err
-            group_id_list = JRecur.get_value_in_forest(group_id_tree)
+            params["user_group_id_list"] = JRecur.get_value_in_forest(group_id_tree)
 
-            user_set = UserToGroup.objects.filter(user_group__in=group_id_list)
-        else:
-            user_set = UserToGroup.objects.filter(user_group=group_id)
+        # 分组下面用户分页查询
+        group_search_params = format_params_handle(
+            param_dict=params,
+            filter_filed_list=["group_id", "user_group_id", "user_group_id_list", "user_id", "user_id_list"],
+            alias_dict={"user_group_id_list": "user_group_id__in", "group_id": "user_group_id", "user_id_list": "user_id__in"}
+        )
+        # 边界查询,分组(部门)必传
+        # if not group_search_params:
+        #     return None, "参数错误,用户部门必选"
+        user_group_set = UserToGroup.objects.filter(**group_search_params).annotate(
+            user_group_value=F("user_group__group_name")
+        ).values("user_id", "user_group_value")
+        total = user_group_set.count()
+        paginator = Paginator(user_group_set, size)
+        # 分页保护
+        try:
+            group_user_list = list(paginator.page(page).object_list)
+        except EmptyPage:
+            group_user_list = list(paginator.page(paginator.num_pages).object_list)
 
-        user_set = user_set.annotate(user_group_value=F("user_group__group_name"))
-        user_set = user_set.values("user_id", "user_group")
-        params = {
-            "page": page,
-            "size": size
-        }
-        # TODO 这样的逻辑 返回的用户组列表 无法显示全部 。解决方案一：在详情的时候处理用户所在的组 方案二：通过聚合表 拿到用户的组
-        user_id_list = [it['user_id'] for it in user_set]
-        if not user_id_list:
-            return {"count": 0, "list": []}, None
-        user_serv, err = UserService.user_list(params, allow_user_list=user_id_list)
-        if err:
-            return None, err
-        return user_serv, None
+        # 拼接用户信息
+        user_serv, err = UserService.user_list(allow_user_list=[it['user_id'] for it in group_user_list], need_Pagination=False)
+        JoinList.left_join(l_list=group_user_list, r_list=user_serv, l_key="user_id", r_key="user_id")
+
+        # 由于数据迁移导致部门绑定的用户ID,用户表存在该用户的信息了,遍历做出提示
+        for i in group_user_list:
+            if i.get("user_name", None) is None:
+                i["email"] = ""
+                i["full_name"] = "该用户不存在,请移除.然后重新绑定."
+                i["user_name"] = "该用户不存在,请移除.然后重新绑定."
+                i["nickname"] = "该用户不存在,请移除.然后重新绑定."
+                i["register_time"] = ""
+                i["wechat_openid"] = ""
+
+        return {"total": total, "page": int(page), "size": int(size), "list": group_user_list}, None
 
     @staticmethod
     def user_bind_group(user_id: int, group_id: int):
@@ -168,10 +223,10 @@ class UserGroupService(object):
         # 如果传控制则表示
         if not group_list:
             UserToGroup.objects.filter(user_id=user_id).delete()
-            return None, None
+            return None, "msg:ok;tip:至少选择一个部门"
 
-        # 因为是批量绑定，所以先删除调已经绑定。然后进行重新插入
         group_list = group_list.split(',') if isinstance(group_list, str) else group_list
+        sid = transaction.savepoint()
         try:
             UserToGroup.objects.filter(user_id=user_id).delete()
             # 添加数据
@@ -181,8 +236,10 @@ class UserGroupService(object):
                     "user_group_id": group
                 }
                 UserToGroup.objects.create(**data)
+            transaction.clean_savepoints()
             return None, None
         except Exception as e:
+            transaction.savepoint_rollback(sid)
             return None, str(e)
 
     @staticmethod
@@ -285,7 +342,7 @@ class UserGroupService(object):
     @staticmethod
     def get_user_group_info(user_id: int = None, user_id_list: list = None, field_list=None, **kwargs):
         """
-        获取用户的分组信息
+        获取指定用户的分组信息
         :param user_id: 用户id
         :param user_id_list: 用户的ID列表
         :param field_list: 字段过滤列表
@@ -325,21 +382,14 @@ class UserGroupService(object):
                     user_group_map[item['user_id']] = [item]
             return user_group_map, None  # 返回映射字典
 
-    # ===============  待修改 start ===============
-    @staticmethod
-    def get_user_from_group(group_id):  # 修改建议，get_user_ids_by_group
-        """
-        根据用户组ID,获取该分组下面的所有用户ID列表
-        :param group_id: 分组ID
-        :return: data, err
-        """
-        user_obj = UserToGroup.objects.filter(user_group_id=group_id)
-        user_list = []
-        if user_obj:
-            user_list = user_obj.values("user_id")
-            user_list = [i['user_id'] for i in user_list]
-        return user_list, None
+    # @staticmethod
+    # def group_user_delete(params):  # 建议删除,应该删除的用户的关系而不是删除这个用户,该操作比较危险
+    #     user_serv, err = UserService.user_delete(params)
+    #     if err:
+    #         return None, err
+    #     return None, None
 
+    # ===============  用户分组和角色关系操作服务 start ===============
     @staticmethod
     def group_user_detail(user_id):  # 修改建议，放在用户模块里面，来角色问角色和分组列表
         from xj_user.services.user_detail_info_service import DetailInfoService
@@ -360,7 +410,7 @@ class UserGroupService(object):
         # 用户角色部门绑定
         user_role_list = params.get('user_role_list', None)
         user_group_list = params.get('user_group_list', None)
-        user_serv, err = UserService.user_add(params)
+        user_serv, err = UserService.user_add(params)  # 为什么要添加用户呢,用户和角色不应该分开吗
         if err:
             return None, err
         if user_group_list:
@@ -379,10 +429,4 @@ class UserGroupService(object):
         RoleService.bind_user_role(params.get("user_id"), user_role_list)
         return None, None
 
-    @staticmethod
-    def group_user_delete(params):  # 建议删除
-        user_serv, err = UserService.user_delete(params)
-        if err:
-            return None, err
-        return None, None
-    # ===============  待修改 start ===============
+    # ===============  用户分组和角色关系操作服务 start ===============
